"""
Copyright 2010, 2011  Leif Theden


This file is part of lib2d.

lib2d is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

lib2d is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with lib2d.  If not, see <http://www.gnu.org/licenses/>.
"""

from buttons import *
from pygame.locals import *
from avatar import Avatar
from lib2d import KeyHandler, UpdateableObject
from vec import Vec2d
from math import radians, atan2
import random

icon_size = (32,32)


def default_icon():
    return None


class GameObject(object):
    """
    the most basic object that can be interacted with in the game.

    icon - this is a standard sized image of the item.

    gameobject[0] will (should) always be the gameworld.
    """

    population = []

    def __init__(self, parent):
        self.icon       = default_icon()
        self.short_name = str(self.__class__)
        self.short_desc = ""
        self.long_desc  = ""
        self._children  = []
        self._parent    = parent
        self.gid        = len(GameObject.population)
        GameObject.population.append(self)

    def serialize(self):
        """
        return a string that represent's this objects state

        children will be serialized as well
        """

        pass

    def get_image(self):
        """
        return an image suitable for drawing onto a surface.

        it is up to the drawing party to know where/how to draw it.  =)
        """

        return self.icon

    def destroy(self, destroyChildren=True):
        """
        destroy the object.  the object will be removed from the game
        and references cleared.  if children are to be removed, they will
        become children of this objects parent.
        """

        if destroy_children:
            for c in self.children:
                c.destroy(True)            
        else:
            for c in self.children:
                c.parent = self.parent

        # remove from population
        self.population.remove(self)

        # we need some kind of check to find if other objects
        # are referencing us now.

class Inventory(GameObject, list):
    """
    This class can hold other objects in "slots"

    subclass of a list, so its cool that way

    a slot specifies an object and a quantity.
    """

    def __init__(self, parent):
        GameObject.__init__(self, parent)
        list.__init__(self)

    def add_slot(self):
        """
        add a new slot to the inventory
        """

        self.append((None, 0))

    def append(self, o):
        o.parent = self
        list.append(self, (0, o))

class Actor(GameObject, UpdateableObject):
    """
    This class represents a game object.

    An actor is updated by the world, and drawn by a view.

    *** Simulation object, physics body, etc ***

    Before adding an actor to the world, the world must
    alreday have a view added, otherwise the actor will never be
    drawn.
    """

    def __init__(self, pos=(10,10)):
        GameObject.__init__(self, None)

        self.avatar = Avatar()
        self.avatar.parent = self

        # inventory for storing items
        self.inv = Inventory(self)

        # this is our position in the world
        self.position = list(pos)

        # used for updates, collisions
        self.old_position = self.position[:]
        
        # im not sure what this is
        self.avatar.position = pos

        # movement vector
        self.v = Vec2d(0,0)
        self._old_angle = 0

        self.collide = None

        self.move_speed = 2

    def get_rect(self):
        """
        used for collision testing
        """

        rect = self.avatar.rect.copy()
        rect.topleft = (self.position[0], self.position[1] + rect.height / 2)
        rect.height = rect.height / 2
        return rect

    def move_to(self, pos):
        self.old_position = self.position[:]
        self.position = list(pos)

    def get_angle(self):
        """
        the angle that the actor is facing

        the result is cached so that direction is retained after actor stops moving.
        this will often be called by an attached avatar when being updated and drawn
        """
        
        if self.v == (0,0):
            return self._old_angle
        else:
            self._old_angle = atan2(self.v[1], self.v[0])
            return self._old_angle
      
    def stop(self):
        """
        stop the actor if moving
        """
        self.v[0] = 0
        self.v[1] = 0
        self.position = self.old_position
 
    def OnPush(self, other):
        pass

    def update(self, time):
        self.avatar.update(time)
        if self.v != (0,0):
            self.move(self.v)

    def move(self, v):
        self.old_position = self.position[:]
        self.position[0] += v[0]
        self.position[1] += v[1]

    def face(self, d):
        angle = {"north": 270, "east": 0, "south": 90, "west": 180}
        self._old_angle = radians(angle[d])

class Hero(Actor):
    """
    Special character that is played by a player
    """

    def attack(self):
        if random.random() >= .5:
            self.avatar.play("attack00")
        else:
            self.avatar.play("attack01")

class ActorGroup(object):
    """
    i'm so ready to be done with this pyweek!
    """

    def __init__(self):
        self.actors = []
    
    def update(self, time):
        [ a.update(time) for a in self.actors ]
    
    def clear(self, surface):
        raise NotImplementedError

    def add(self, obj):
        if isinstance(obj, Actor):
            self.actors.append(obj)
            return True
        else:
            return False
